<?php
// $Id: ac.inc,v 1.3 2008/08/15 20:50:15 tjholowaychuk Exp $  

/**
 * @file 
 * Ajax callback controller API.
 * @author Tj Holowaychuk <tj@vision-media.ca>
 * @link http://vision-media.ca
 * @package ac
 */
 
/**
 * Process the AC callback.
 * 
 * @param string $module
 * 
 * @param string $op
 */
function ac_process_request($module, $op) {
  $func = "{$module}_js_{$op}";
       
  // Ensure callback exists
  if (!function_exists($func)){
    watchdog('ac', 'Invalid callback function @function', array('@function' => $func), WATCHDOG_NOTICE);
    die(t('Invalid callback.'));
  }
  
  // Validate token
  if (!isset($_GET['token']) || !_ac_check_token($_GET['token'])){
    watchdog('ac', 'Invalid token');
    die(t('Callback failed.'));
  }
  
  // Build args from query string
  $args = $_GET;
  unset($args['q']);   

  // Execute callback   
  $response = _ac_get_default_response();
  $func($response, $args);
  
  // Allow modules to alter the response
  drupal_alter('js_response', $response, $module, $op);     
  
  // Output
  _ac_output($response);
}

/**
 * Debug a variable.
 * 
 * This function outputs PHP data to the Firebug console which
 * is useful for quickly debugging XMLHttpRequests.  
 * 
 * @param mixed $var
 */
function ac_debug($var) {  
  $var = drupal_to_js($var);
  
  $js  = "console.group('Debug');";
  $js .= "console.log({$var});";
  $js .= "console.groupEnd('Debug');";
      
  drupal_json(array('debug' => $js));
  exit();
}

/**
 * Build callback output.
 * 
 * @param array $response
 * 
 * @see ac_process_request()
 */
function _ac_output($response) {
  $output = array();
  
  // Assign error message when an error has occurred
  if ($response['status'] == AC_STATUS_ERROR){ 
    if (empty($response['message'])){
      $response['message'] = AC_STATUS_ERROR_MESSAGE;
    }
  }
  
  // Add response to output
  // @todo remove arbitrary members
  $output = $response;
  
  // Format
  // Currently only supporting JSON
  $json = drupal_to_js($output);
  drupal_set_header('Content-Type: text/javascript; charset=utf-8'); 
 
  // Gzip output
  if (AC_ENCODING_MAY_COMPRESS && $_SERVER['HTTP_ACCEPT_ENCODING'] == 'gzip,deflate'){
    if ($response['encoding'] == AC_ENCODING_GZIP || ($response['encoding'] == AC_ENCODING_AUTO && _ac_check_length($json))){ 
      if (function_exists('gzencode')){
        drupal_set_header('Content-Encoding: gzip');
        $json = gzencode($json);
      }  
    }                                      
  }
  
  // Set headers passed
  foreach((array) $response['headers'] AS $header){
    drupal_set_header($header);
  }
  
  // Output content
  drupal_set_header('Content-Length: ' . strlen($json));
  echo $json;
  exit();
}        

/**
 * Validate the AC token passed.
 * 
 * @param string $token
 * 
 * @return bool
 * 
 * @todo use drupal_valid_token().
 */
function _ac_check_token($token){
  if (isset($_SESSION['ac_token']) && $_SESSION['ac_token'] == $token){
    return TRUE;
  }
  
  return FALSE; 
}

/**
 * Validate weither or not output should be 
 * compressed based on the length of output.
 * 
 * @param string $string
 * 
 * @return bool
 */
function _ac_check_length($string){
  if (strlen($string) >= AC_ENCODING_AUTO_THRESHOLD){
    return TRUE;
  }
  
  return FALSE;
}

/**
 * Get default response structure.
 * 
 * @return array
 */
function _ac_get_default_response() {
  // @todo allow usage of message_func = 'append' etc
  // @todo allow message_effect = slide OR fade etc
  return array(
      // Request status
      'status' => AC_STATUS_OK,   
      // Message wrapped with t().
      // This member is context sensative and will display an error
      // when status is AC_STATUS_ERROR. When no error message is supplied
      // variable_get('ac_error_message') is used.
      'message' => '',                          
      // CSS Selector used to place messages
      'message_wrapper' => '#ac-messages', 
      // Message display duration in milliseconds 
      'message_duration' => 3000,   
      // Additional output, markup, array, object, etc
      'output' => NULL,         
      // Output encoding such as gzip or no plain-text
      'encoding' => AC_ENCODING_AUTO, 
      // Output format. Currently supports JSON only, if you
      // are in need of elaborate output formats view the server module
      'format' => AC_OUTPUT_JSON,   
      // Additional response headers. 
      'headers' => array(),   
    );
}
              
 